import { BatchGetCommandInput, DeleteCommandInput, GetCommandInput, PutCommandInput, QueryCommandInput, ScanCommandInput } from '@aws-sdk/lib-dynamodb';
import { ConditionExpressionPredicate, ConditionExpression } from '@aws/dynamodb-expressions';
import { IndexName, Storage } from './storage';
import { IPageToken } from './page-token';
export interface KeyConfig {
    keys: string[];
    delimiter: string;
}
export declare type KeyFactory<T> = (entity: T) => string | undefined;
export declare type KeyDefinition<T> = string | string[] | KeyConfig | KeyFactory<T>;
export interface IIndexDefinition<T> {
    hash: KeyDefinition<T>;
    sort?: KeyDefinition<T>;
}
export interface IKeyGenerator<T> {
    hash: KeyFactory<T>;
    sort?: KeyFactory<T>;
}
export declare type ISchemaDefinition<T> = Record<IndexName, IIndexDefinition<T>>;
export interface IRepositoryParams<T> {
    storage: Storage;
    schema: Partial<ISchemaDefinition<T>>;
}
export interface IQueryParams<T = undefined> {
    limit: number;
    indexName: IndexName;
    keyParams?: Partial<T>;
    hash?: string | number;
    sort?: string | number | ConditionExpressionPredicate;
    startKey?: IPageToken;
    filter?: ConditionExpression;
}
export interface IScanParams {
    indexName?: IndexName;
    startKey?: IPageToken;
    limit: number;
    filter?: ConditionExpression;
}
export declare class Schema<T> {
    storage: Storage;
    schema: Partial<Record<IndexName, IKeyGenerator<T>>>;
    constructor(params: IRepositoryParams<T>);
    indexData(entity: Partial<T>, indexName?: IndexName): Record<string, any>;
    putParams(entity: T): PutCommandInput;
    getParams(keyParams: Partial<T>): GetCommandInput;
    batchGetParams(keyParams: Partial<T>[]): BatchGetCommandInput;
    deleteParams(keyParams: Partial<T>): DeleteCommandInput;
    queryParams({ indexName, keyParams, hash, sort, filter, startKey, limit, }: IQueryParams<T>): QueryCommandInput;
    scanParams({ indexName, startKey, limit, filter, }: IScanParams): ScanCommandInput;
}
export declare function conditionToExpression(condition: Record<string, any>): string;
export declare function conditionsToValues(condition: Record<string, any>): {
    [k: string]: any;
};
